% optithgen 
% optimal thermal generation mix using screening curves
% Julio Usaola
% Universidad Carlos III de Madrid  - 2020

clearvars
close all
clc
format compact

nhoras=8760;  % hours in a year
dataequil0   %
% data_equil

nclases=8760; % time intervals to be considered - reduce only if there are not enough memory
horas=zeros(nclases,1);
dclases=max(monot)-(0:nclases-1)*(max(monot)-min(monot))/nclases;
for iclases=1:nclases
    horas(iclases)=abc(find(monot<=dclases(iclases),1))*nhoras;
end
dhoras=[diff(horas);nhoras+1-horas(length(horas))]/nhoras;

ntec=length(fc);

fvec=zeros((nclases+1)*ntec,1);
aeq=zeros(nclases,(nclases+1)*ntec);
adeq=zeros(ntec*nclases,(nclases+1)*ntec);
bdeq=zeros(nclases*ntec,1);
lb=zeros((nclases+1)*ntec,1);
ub=inf((nclases+1)*ntec,1);

for itec=1:ntec
    fvec((itec-1)*nclases+1:itec*nclases)=vc(itec)*dhoras;
    fvec(ntec*nclases+itec)=fc(itec);
    aeq(:,(itec-1)*nclases+1:itec*nclases)=-eye(nclases);
    adeq((itec-1)*nclases+1:itec*nclases,(itec-1)*nclases+1:itec*nclases)=...
        eye(nclases);
    adeq((itec-1)*nclases+1:itec*nclases,nclases*ntec+itec)=-ones(nclases,1);
end

[x,fval,exitflag,output,lambda]=linprog(fvec,adeq,bdeq,aeq,-dclases,lb,ub);

pinst=x(ntec*nclases+1:length(x));
energ=zeros(ntec,nclases);
for itec=1:ntec
    nini=(itec-1)*nclases;
    nfin=nini+nclases;
    energ(itec,:)=x(nini+1:nfin).*dhoras*8760;
end
energ_tot=sum(energ,2);
energ_sum=sum(energ_tot);
disp('Total installed thermal power')
disp('     peak - middle - base (GW)')
disp(pinst')
disp(sum(pinst))
disp('Energy produced by each technology')
disp('     peak - middle - base (TWh)')
disp(energ_tot'/1000)
disp('Total energy (TWh)- check')
disp(energ_sum/1000)
disp('Original demand (TWh) - check')
disp(sum(monot)/1000)

save cartera.mat energ






